package openio

import (
	"context"
	"fmt"
	"os"
	"strconv"
	"strings"
	"syscall"
	"time"

	"github.com/open-io/oio-exporter/common"
	"github.com/open-io/oio-exporter/daemon"
)

type openioVolumeDaemonParams struct {
	Namespace     string
	ServiceType   string
	ServiceID     string
	ServiceIP     string
	ServicePort   int
	ServiceVolume string
}

type openioVolumeDaemon struct {
	daemon.BaseDaemon

	namespace     string
	serviceType   string
	serviceID     string
	serviceIP     string
	servicePort   int
	serviceVolume string
	serviceAddr   string
}

func (d *openioVolumeDaemon) Interval() time.Duration {
	return confInterval
}

func (d *openioVolumeDaemon) Init(ctx context.Context) error {
	d.serviceAddr = fmt.Sprintf("%s:%d", d.serviceIP, d.servicePort)

	return nil
}

func (d *openioVolumeDaemon) Collect(ctx context.Context, m *common.MetricData) error {
	serviceType := d.serviceType
	serviceVolume := d.serviceVolume

	f, err := os.OpenFile(serviceVolume, os.O_RDONLY, 0644)
	if err != nil {
		m.IncrError("openio", serviceType)
		return err
	}
	defer f.Close()

	var stat syscall.Statfs_t
	err = syscall.Fstatfs(int(f.Fd()), &stat)
	if err != nil {
		m.IncrError("openio", serviceType)
		return err
	}
	vMetric := map[string]uint64{
		"size_bytes/avail": stat.Bavail * uint64(stat.Bsize),
		"size_bytes/used":  (stat.Blocks - stat.Bfree) * uint64(stat.Bsize),
		"size_bytes/free":  stat.Bfree * uint64(stat.Bsize),
		"inodes/free":      stat.Ffree,
		"inodes/used":      stat.Files - stat.Ffree,
	}
	fsID := strconv.FormatUint(uint64(stat.Fsid.X__val[1])<<32|uint64(stat.Fsid.X__val[0]), 10)
	for dim, val := range vMetric {
		vars := strings.Split(dim, "/")
		if len(vars) != 2 {
			continue
		}

		m.Set(&common.Metric{
			Name: "volume_" + vars[0],
			Labels: map[string]string{
				"namespace":    d.namespace,
				"service":      d.serviceID,
				"service_type": serviceType,
				"service_addr": d.serviceAddr,
				"volume":       serviceVolume,
				"fsid":         fsID,
				"type":         vars[1],
			},
			Value: float64(val),
			TTL:   confInterval * 2,
		})
	}

	return nil
}
